(function() {
  var Akismet, http, qs, util;
  http = require('http');
  qs = require('querystring');
  util = require('util');
  Akismet = function(options) {
    this.blog = options.blog || '';
    this.apiKey = options.apiKey;
    this.host = options.host || 'rest.akismet.com';
    this.endPoint = options.endPoint || this.apiKey + '.' + this.host;
    this.port = options.port || 80;
    this.userAgent = options.userAgent || 'Generic Node.js/1.0.0 | Akismet 2.4.0';
    this.charset = options.charset || 'utf-8';
    this.debug = options.debug || false;
    return this;
  };
  Akismet.prototype.verifyKey = function(callback) {
    return this.postRequest(this.host, '/1.1/verify-key', {
      key: this.apiKey,
      blog: this.blog
    }, function(status, headers, body) {
      return callback((status >= 200) && status < 300 && body === 'valid' ? true : false, status, headers);
    });
  };
  Akismet.prototype.checkSpam = function(options, callback) {
    options.blog = this.blog;
    options.user_agent = this.userAgent;
    return this.postRequest(this.endPoint, '/1.1/comment-check', options, function(status, headers, body) {
      return callback((status >= 200) && status < 300 && body === 'true' ? true : false, status, headers);
    });
  };
  Akismet.prototype.submitSpam = function(options, callback) {
    options.blog = this.blog;
    options.user_agent = this.userAgent;
    return this.postRequest(this.endPoint, '/1.1/submit-spam', options, function(status, headers, body) {
      return callback(status, headers);
    });
  };
  Akismet.prototype.submitHam = function(options, callback) {
    options.blog = this.blog;
    options.user_agent = this.userAgent;
    return this.postRequest(this.endPoint, '/1.1/submit-ham', options, function(status, headers, body) {
      return callback(status, headers);
    });
  };
  Akismet.prototype.postRequest = function(host, path, query, callback) {
    var client, req;
    query = qs.stringify(query);
    if (this.debug) {
      util.log('Sending request: ' + host + ':' + this.port);
      util.log('Request body: ' + query);
    }
    client = http.createClient(this.port, host);
    req = client.request('POST', path, {
      'host': host,
      'content-type': 'application/x-www-form-urlencoded; charset=' + this.charset,
      'content-length': query.length,
      'user-agent': this.userAgent
    });
    req.write(query);
    req.end();
    return req.on('response', function(res) {
      if (this.debug) {
        util.log('Response: ' + util.inspect(res));
      }
      res.setEncoding('utf-8');
      return res.on('data', function(body) {
        return callback(res.statusCode, res.headers, body);
      });
    });
  };
  module.exports = {
    client: function(options) {
      return new Akismet(options);
    }
  };
}).call(this);
